@@ -1,15 +1,13 @@
 //! Tests for the `cargo package` command.
 
-use std::fs::{read_to_string, File};
-use std::io::prelude::*;
-use std::path::Path;
-
 use cargo_test_support::paths::CargoPathExt;
-use cargo_test_support::registry::Package;
+use cargo_test_support::publish::validate_crate_contents;
+use cargo_test_support::registry::{self, Package};
 use cargo_test_support::{
-    basic_manifest, cargo_process, git, path2url, paths, project, publish::validate_crate_contents,
-    registry, symlink_supported, t,
+    basic_manifest, cargo_process, git, path2url, paths, project, symlink_supported, t,
 };
+use std::fs::{self, read_to_string, File};
+use std::path::Path;
 
 #[cargo_test]
 fn simple() {
@@ -560,18 +558,10 @@ fn package_symlink_to_submodule() {
 
 #[cargo_test]
 fn no_duplicates_from_modified_tracked_files() {
-    let root = paths::root().join("all");
-    let p = git::repo(&root)
-        .file("Cargo.toml", &basic_manifest("foo", "0.0.1"))
-        .file("src/main.rs", "fn main() {}")
-        .build();
-    File::create(p.root().join("src/main.rs"))
-        .unwrap()
-        .write_all(br#"fn main() { println!("A change!"); }"#)
-        .unwrap();
-    cargo_process("build").cwd(p.root()).run();
-    cargo_process("package --list --allow-dirty")
-        .cwd(p.root())
+    let p = git::new("all", |p| p.file("src/main.rs", "fn main() {}"));
+    p.change_file("src/main.rs", r#"fn main() { println!("A change!"); }"#);
+    p.cargo("build").run();
+    p.cargo("package --list --allow-dirty")
         .with_stdout(
             "\
 Cargo.lock
@@ -669,17 +659,7 @@ fn repackage_on_source_change() {
     p.cargo("package").run();
 
     // Add another source file
-    let mut file = File::create(p.root().join("src").join("foo.rs")).unwrap_or_else(|e| {
-        panic!(
-            "could not create file {}: {}",
-            p.root().join("src/foo.rs").display(),
-            e
-        )
-    });
-
-    file.write_all(br#"fn main() { println!("foo"); }"#)
-        .unwrap();
-    std::mem::drop(file);
+    p.change_file("src/foo.rs", r#"fn main() { println!("foo"); }"#);
 
     // Check that cargo rebuilds the tarball
     p.cargo("package")
@@ -874,18 +854,7 @@ fn generated_manifest() {
 
     let f = File::open(&p.root().join("target/package/foo-0.0.1.crate")).unwrap();
     let rewritten_toml = format!(
-        r#"# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
-#
-# When uploading crates to the registry Cargo will automatically
-# "normalize" Cargo.toml files for maximal compatibility
-# with all versions of Cargo and also rewrite `path` dependencies
-# to registry (e.g., crates.io) dependencies
-#
-# If you believe there's an error in this file please file an
-# issue against the rust-lang/cargo repository. If you're
-# editing this file be aware that the upstream Cargo.toml
-# will likely look very different (and much more reasonable)
-
+        r#"{}
 [package]
 name = "foo"
 version = "0.0.1"
@@ -909,6 +878,7 @@ registry-index = "{}"
 [dependencies.ghi]
 version = "1.0"
 "#,
+        cargo::core::package::MANIFEST_PREAMBLE,
         registry::alt_registry_url()
     );
 
@@ -955,28 +925,20 @@ fn ignore_workspace_specifier() {
     p.cargo("package --no-verify").cwd("bar").run();
 
     let f = File::open(&p.root().join("target/package/bar-0.1.0.crate")).unwrap();
-    let rewritten_toml = r#"# THIS FILE IS AUTOMATICALLY GENERATED BY CARGO
-#
-# When uploading crates to the registry Cargo will automatically
-# "normalize" Cargo.toml files for maximal compatibility
-# with all versions of Cargo and also rewrite `path` dependencies
-# to registry (e.g., crates.io) dependencies
-#
-# If you believe there's an error in this file please file an
-# issue against the rust-lang/cargo repository. If you're
-# editing this file be aware that the upstream Cargo.toml
-# will likely look very different (and much more reasonable)
-
+    let rewritten_toml = format!(
+        r#"{}
 [package]
 name = "bar"
 version = "0.1.0"
 authors = []
-"#;
+"#,
+        cargo::core::package::MANIFEST_PREAMBLE
+    );
     validate_crate_contents(
         f,
         "bar-0.1.0.crate",
         &["Cargo.toml", "Cargo.toml.orig", "src/lib.rs"],
-        &[("Cargo.toml", rewritten_toml)],
+        &[("Cargo.toml", &rewritten_toml)],
     );
 }
 
@@ -1086,6 +1048,38 @@ Caused by:
         .run();
 }
 
+#[cargo_test]
+fn test_edition_from_the_future() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"[package]
+                edition = "2038"
+                name = "foo"
+                version = "99.99.99"
+                authors = []
+            "#,
+        )
+        .file("src/main.rs", r#""#)
+        .build();
+
+    p.cargo("build")
+        .with_status(101)
+        .with_stderr(
+            "\
+error: failed to parse manifest at `[..]`
+
+Caused by:
+  failed to parse the `edition` key
+
+Caused by:
+  this version of Cargo is older than the `2038` edition, and only supports `2015` and `2018` editions.
+"
+            .to_string(),
+        )
+        .run();
+}
+
 #[cargo_test]
 fn do_not_package_if_src_was_modified() {
     let p = project()
@@ -1663,3 +1657,191 @@ src/lib.rs
     let orig = read_to_string(p.root().join("target/package/foo-1.0.0/Cargo.toml.orig")).unwrap();
     assert!(orig.contains("license-file = \"../LICENSE\""));
 }
+
+#[cargo_test]
+#[cfg(not(windows))] // Don't want to create invalid files on Windows.
+fn package_restricted_windows() {
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            license = "MIT"
+            description = "foo"
+            homepage = "foo"
+            "#,
+        )
+        .file("src/lib.rs", "pub mod con;\npub mod aux;")
+        .file("src/con.rs", "pub fn f() {}")
+        .file("src/aux/mod.rs", "pub fn f() {}")
+        .build();
+
+    p.cargo("package")
+        .with_stderr(
+            "\
+[WARNING] file src/aux/mod.rs is a reserved Windows filename, it will not work on Windows platforms
+[WARNING] file src/con.rs is a reserved Windows filename, it will not work on Windows platforms
+[PACKAGING] foo [..]
+[VERIFYING] foo [..]
+[COMPILING] foo [..]
+[FINISHED] [..]
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn finds_git_in_parent() {
+    // Test where `Cargo.toml` is not in the root of the git repo.
+    let repo_path = paths::root().join("repo");
+    fs::create_dir(&repo_path).unwrap();
+    let p = project()
+        .at("repo/foo")
+        .file("Cargo.toml", &basic_manifest("foo", "0.1.0"))
+        .file("src/lib.rs", "")
+        .build();
+    let repo = git::init(&repo_path);
+    git::add(&repo);
+    git::commit(&repo);
+    p.change_file("ignoreme", "");
+    p.change_file("ignoreme2", "");
+    p.cargo("package --list --allow-dirty")
+        .with_stdout(
+            "\
+Cargo.toml
+Cargo.toml.orig
+ignoreme
+ignoreme2
+src/lib.rs
+",
+        )
+        .run();
+
+    p.change_file(".gitignore", "ignoreme");
+    p.cargo("package --list --allow-dirty")
+        .with_stdout(
+            "\
+.gitignore
+Cargo.toml
+Cargo.toml.orig
+ignoreme2
+src/lib.rs
+",
+        )
+        .run();
+
+    fs::write(repo_path.join(".gitignore"), "ignoreme2").unwrap();
+    p.cargo("package --list --allow-dirty")
+        .with_stdout(
+            "\
+.gitignore
+Cargo.toml
+Cargo.toml.orig
+src/lib.rs
+",
+        )
+        .run();
+}
+
+#[cargo_test]
+#[cfg(windows)]
+fn reserved_windows_name() {
+    Package::new("bar", "1.0.0")
+        .file("src/lib.rs", "pub mod aux;")
+        .file("src/aux.rs", "")
+        .publish();
+
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [project]
+            name = "foo"
+            version = "0.0.1"
+            authors = []
+            license = "MIT"
+            description = "foo"
+
+            [dependencies]
+            bar = "1.0.0"
+        "#,
+        )
+        .file("src/main.rs", "extern crate bar;\nfn main() {  }")
+        .build();
+    p.cargo("package")
+        .with_status(101)
+        .with_stderr_contains(
+            "\
+error: failed to verify package tarball
+
+Caused by:
+  failed to download replaced source registry `[..]`
+
+Caused by:
+  failed to unpack package `[..] `[..]`)`
+
+Caused by:
+  failed to unpack entry at `[..]aux.rs`
+
+Caused by:
+  `[..]aux.rs` appears to contain a reserved Windows path, it cannot be extracted on Windows
+
+Caused by:
+  failed to unpack `[..]aux.rs`
+
+Caused by:
+  failed to unpack `[..]aux.rs` into `[..]aux.rs`",
+        )
+        .run();
+}
+
+#[cargo_test]
+fn list_with_path_and_lock() {
+    // Allow --list even for something that isn't packageable.
+
+    // Init an empty registry because a versionless path dep will search for
+    // the package on crates.io.
+    registry::init();
+    let p = project()
+        .file(
+            "Cargo.toml",
+            r#"
+            [package]
+            name = "foo"
+            version = "0.1.0"
+            license = "MIT"
+            description = "foo"
+            homepage = "foo"
+
+            [dependencies]
+            bar = {path="bar"}
+            "#,
+        )
+        .file("src/main.rs", "fn main() {}")
+        .file("bar/Cargo.toml", &basic_manifest("bar", "0.1.0"))
+        .file("bar/src/lib.rs", "")
+        .build();
+
+    p.cargo("package --list")
+        .with_stdout(
+            "\
+Cargo.lock
+Cargo.toml
+Cargo.toml.orig
+src/main.rs
+",
+        )
+        .run();
+
+    p.cargo("package")
+        .with_status(101)
+        .with_stderr(
+            "\
+error: all path dependencies must have a version specified when packaging.
+dependency `bar` does not specify a version.
+",
+        )
+        .run();
+}
